@@ -2,32 +2,8 @@
  * Like what you see? Join us!
  * https://www.univention.com/about-us/careers/vacancies/
  *
- * Copyright 2011-2025 Univention GmbH
- *
- * https://www.univention.de/
- *
- * All rights reserved.
- *
- * The source code of this program is made available
- * under the terms of the GNU Affero General Public License version 3
- * (GNU AGPL V3) as published by the Free Software Foundation.
- *
- * Binary versions of this program provided by Univention to you as
- * well as other copyrighted, protected or trademarked materials like
- * Logos, graphics, fonts, specific documentations and configurations,
- * cryptographic keys etc. are subject to a license agreement between
- * you and Univention and not subject to the GNU AGPL V3.
- *
- * In the case you use this program under the terms of the GNU AGPL V3,
- * the program is provided in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Affero General Public License for more details.
- *
- * You should have received a copy of the GNU Affero General Public
- * License with the Debian GNU/Linux or Univention distribution in file
- * /usr/share/common-licenses/AGPL-3; if not, see
- * <https://www.gnu.org/licenses/>.
+ * SPDX-FileCopyrightText: 2011-2025 Univention GmbH
+ * SPDX-License-Identifier: AGPL-3.0-only
  */
 /*global require,define,setTimeout,window,console*/
 
@@ -228,14 +204,33 @@ define([
 			// for the detail page, we first need to query property data from the server
 			// for the layout of the selected object type, then we can render the page
 			var objectDN = this._multiEdit || this.moduleFlavor == 'users/self' || this.ldapName instanceof Deferred ? null : this.ldapName || null;
+
+			var effectiveObjectDN = objectDN;
+			var anObjectIsBeingCreated = !effectiveObjectDN && this.newObjectOptions;
+			if (anObjectIsBeingCreated) {
+				var containerDN = lang.getObject('container', false, this.newObjectOptions);
+				if (containerDN) {
+					effectiveObjectDN = containerDN;
+				}
+			}
+
 			// prepare parallel queries
 			var moduleCache = cache.get(this.moduleFlavor);
-			this.propertyQuery = moduleCache.getProperties(this.objectType, objectDN);
+
+			// Use effectiveObjectDN for fetching properties and layout
+			// For new objects with a determined container, force load to ensure OU-specific defaults are fresh.
+			if (anObjectIsBeingCreated && effectiveObjectDN) {
+				this.propertyQuery = moduleCache.getProperties(this.objectType, effectiveObjectDN, true); // forceLoad = true
+			} else {
+				this.propertyQuery = moduleCache.getProperties(this.objectType, effectiveObjectDN);
+			}
+
 			var commands = {
 				properties: this.propertyQuery,
 				layout: moduleCache.getLayout(this.objectType, objectDN),
 				metaInfo: moduleCache.getMetaInfo(this.objectType)
 			};
+
 			if (!this._multiEdit) {
 				// query policies for normal edit
 				commands.policies = moduleCache.getPolicies(this.objectType);
