@@ -24,6 +24,7 @@ from urllib.request import Request
 
 from ldap import INVALID_CREDENTIALS, LDAPError
 
+import univention.admin.authorization as udm_auth
 import univention.admin.modules as udm_modules
 import univention.admin.objects as udm_objects
 import univention.admin.syntax as udm_syntax
@@ -33,7 +34,8 @@ import univention.directory.reports as udr
 from univention.config_registry import handler_set
 from univention.lib.i18n import Translation
 from univention.management.console.config import ucr
-from univention.management.console.ldap import get_user_connection
+from univention.management.console.error import Forbidden
+from univention.management.console.ldap import get_admin_connection as get_ldap_admin_connection, get_user_connection
 from univention.management.console.log import MODULE
 from univention.management.console.message import Request as UMCRequest
 from univention.management.console.modules import Base, UMC_Error
@@ -52,7 +54,7 @@ from .udm_ldap import (
     LDAP_AuthenticationFailed, NoIpLeft, ObjectDoesNotExist, SuperordinateDoesNotExist, UDM_Error, UDM_Module,
     UserWithoutDN, _get_syntax, calculate_bind_hash, container_modules, get_bind_hash, get_module, get_obj_module,
     info_syntax_choices, ldap_dn2path, list_objects, read_syntax_choices, search_syntax_choices_by_key,
-    set_bind_function, set_bind_hash, set_user_roles,
+    set_bind_function, set_bind_hash,
 )
 
 
@@ -184,13 +186,14 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
 
         def bind_user_connection(lo):
             request.bind_user_connection(lo)
+            lo = udm_auth.Authorization.inject_ldap_connection(lo)
             self.require_license(lo)
 
         set_bind_function(bind_user_connection)
         set_bind_hash(calculate_bind_hash(request))
 
-        if ucr.is_true("umc/udm/delegation"):
-            set_user_roles(request.user_dn)
+        if ucr.is_true("directory/manager/web/delegative-administration/enabled"):
+            udm_auth.Authorization.enable(lambda: get_ldap_admin_connection()[0])  # noqa: PLW0108
 
         # read user settings and initial UDR
         self.reports_cfg = udr.Config()
@@ -205,6 +208,8 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
         if isinstance(exc, udm_errors.authFail | INVALID_CREDENTIALS):
             MODULE.warn('Authentication failed: %s' % (exc,))
             raise LDAP_AuthenticationFailed()
+        if isinstance(exc, udm_errors.permissionDenied) or isinstance(exc, UDM_Error) and isinstance(exc.exc, udm_errors.permissionDenied):
+            raise Forbidden(str(exc))
         if isinstance(exc, udm_errors.base | LDAPError):
             MODULE.error(''.join(traceback.format_exception(etype, exc, etraceback)))
 
@@ -221,15 +226,14 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
         except LicenseError:
             lo.allow_modify = False
         lo.requireLicense()
+        lo.authz_connection.requireLicense()
 
     def get_ldap_connection(self):
-        if ucr.is_true("umc/udm/delegation"):
-            from univention.management.console.ldap import get_admin_connection as ldap_get_admin_connection
-            return ldap_get_admin_connection()
         try:
             lo, _po = get_user_connection(bind=self.bind_user_connection, write=True, bindhash=calculate_bind_hash(self._current_request))
         except (LDAPError, udm_errors.ldapError):
             lo, _po = get_user_connection(bind=self.bind_user_connection, write=True, bindhash=calculate_bind_hash(self._current_request))
+        lo = udm_auth.Authorization.inject_ldap_connection(lo)
         return lo, udm_uldap.position(lo.base)
 
     def get_module(self, flavor, ldap_dn):
@@ -528,7 +532,7 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
                     obj.set_defaults = True
                     obj.set_default_values()
                     _remove_uncopyable_properties(obj)
-                    props = obj.info
+
                     empty_props_with_default_set = {}
                     for key in obj.info.keys():
                         if obj.hasChanged(key):
@@ -536,6 +540,15 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
                                 'default_value': obj.info[key],
                                 'prevent_umc_default_popup': obj.descriptions[key].prevent_umc_default_popup,
                             }
+
+                    # show all lazy loading properties in UMC
+                    lazy_loading_props = {key: obj.descriptions[key] for key in obj.descriptions if (obj.has_property(key)) and obj.descriptions[key].lazy_loading_fn}
+                    for prop in lazy_loading_props.values():
+                        prop.lazy_load(obj)
+
+                    obj.authz.filter_object_properties(obj)
+
+                    props = obj.info
                     props['$empty_props_with_default_set$'] = empty_props_with_default_set
 
                     for passwd in module.password_properties:
@@ -556,11 +569,6 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
                     props['$operations$'] = module.operations
                     props['$references$'] = module.get_policy_references(ldap_dn)
 
-                    # show all lazy loading properties in UMC
-                    lazy_loading_props = {key: obj.descriptions[key] for key in obj.descriptions if (obj.has_property(key)) and obj.descriptions[key].lazy_loading_fn}
-                    for prop in lazy_loading_props.values():
-                        prop.lazy_load(obj)
-
                     result.append(props)
                 else:
                     MODULE.process('The LDAP object for the LDAP DN %s could not be found' % ldap_dn)
@@ -629,7 +637,7 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
         fields = (set(request.options.get('fields', []) or []) | {objectProperty}) - {'name', 'None'}
         result = []
         for container in containers:
-            result.extend(module.search(container, objectProperty, objectPropertyValue, superordinate, scope=scope, hidden=hidden, allow_asterisks=USE_ASTERISKS))
+            result.extend(module.search(container, objectProperty, objectPropertyValue, superordinate, scope=scope, hidden=hidden, allow_asterisks=USE_ASTERISKS) or [])
 
         entries = []
         object_type = request.options.get('objectType', request.flavor)
@@ -663,7 +671,14 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
         """Returns a list of reports for the given object type"""
         # i18n: translation for univention-directory-reports
         _('PDF Document')
-        self.finished(request.id, [{'id': name, 'label': _(name)} for name in sorted(self.reports_cfg.get_report_names(request.flavor))])
+        ldap_connection = self.get_ldap_connection()[0]
+        report_types = self.reports_cfg.get_report_names(request.flavor)
+        result = [
+            {'id': name, 'label': _(name)}
+            for name in sorted(report_types)
+            if ldap_connection.authz.is_report_create_allowed(ldap_connection, request.flavor, name, raise_exception=False)
+        ]
+        self.finished(request.id, result)
 
     def sanitize_reports_create(self, request):
         choices = self.reports_cfg.get_report_names(request.flavor)
@@ -677,6 +692,7 @@ class Instance(Base, ProgressMixin, metaclass=UDMModuleMeta):
     def reports_create(self, request):
         """Creates a report for the given LDAP DNs and returns the URL to access the file"""
         ldap_connection = self.get_ldap_connection()[0]
+        ldap_connection.authz.is_report_create_allowed(ldap_connection, request.flavor, request.options['report'])
         report = udr.Report(ldap_connection)
         try:
             report_file = report.create(request.flavor, request.options['report'], request.options['objects'])
